/**************************************************************** 
 * Description: 
 getmin 函数：用于在指定区间 [l, r] 中找到最小值的索引，这个最小值将作为当前子树的根节点。
dfs 函数：使用深度优先搜索来构建笛卡尔树，递归处理当前区间的左右子树，并将每个节点值按层次存入 s 向量中。
主函数 main：读取输入的数组大小和元素，调用 dfs 函数构建笛卡尔树，最后输出树的层序遍历。
 * Author: Alex Li
 * Date: 2024-08-20 01:05:03
 * LastEditTime: 2024-08-20 01:32:59
****************************************************************/

#include<iostream>
#include<vector>
using namespace std;

const int N = 40; // 定义常量N，用于限制数组和向量的最大大小
vector<int> s[N]; // s数组存储各个深度的节点，用于层次遍历的输出
int w[N]; // w数组存储输入的序列

// 在区间 [l, r] 中找到最小值的索引
int getmin(int l, int r) {
    int res = l; // 初始时将结果设置为区间的起始位置
    for (int i = l; i <= r; i++) { // 遍历区间 [l, r]
        if (w[res] > w[i]) // 如果找到更小的值，更新最小值的索引
            res = i;
    }
    return res; // 返回区间内最小值的索引
}

// 使用深度优先搜索 (DFS) 构建笛卡尔树，并记录每层的节点
void dfs(int l, int r, int d) {
    if (l > r) return; // 递归边界，当区间无效时结束递归
    int root = getmin(l, r); // 找到当前区间的最小值索引作为根节点
    s[d].push_back(w[root]); // 将根节点的值存入当前深度的向量
    dfs(l, root - 1, d + 1); // 递归处理左子树
    dfs(root + 1, r, d + 1); // 递归处理右子树
}

int main() {
    int n;
    cin >> n; // 读取数组的大小
    for (int i = 0; i < n; i++)
        cin >> w[i]; // 读取数组元素
    dfs(0, n - 1, 0); // 调用DFS函数构建笛卡尔树
    for (int i = 0; s[i].size(); i++) // 遍历每一层
        for (auto x : s[i]) // 输出每一层的节点值
            cout << x << ' ';
    return 0;
}
